/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.render;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Polygon;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.List;

/**
 *
 * @author johan
 */
public class SplitPolygonDrawingRenderer implements DrawingRenderer {
    private final List<Coordinate> coords;
    private final Polygon source;
    private final List<Polygon> results;

    /**
     * 
     * @param coords drawn splitline coordinates
     * @param source geometry to split
     * @param results split result geometries
     */
    public SplitPolygonDrawingRenderer(List<Coordinate> coords, Polygon source, List<Polygon> results) {
        this.coords = coords;
        this.source = source;
        this.results = results;
    }

    @Override
    public void render(Graphics2D g, Rectangle canvas, GeoTranslator translator) {
        
        // show either the split results when available (shades of green),
        // or the source geometry to split (red):
        if (results.size() == 2) {
            renderGeometry(g, results.get(0), new Color(0, 255, 127, 127), canvas, translator);
            renderGeometry(g, results.get(1), new Color(127, 255, 0, 127), canvas, translator);
        }
        else {
            renderGeometry(g, source, new Color(255, 0, 0, 127), canvas, translator);
        }

        // show the splitline:
        if (coords != null && !coords.isEmpty()) {
            
            int len = coords.size();
            int xs[] = new int[len];
            int ys[] = new int[len];
            
            // TODO: calculate bounding box, 
            // no problems yet since the currently used translator doesn't need it.
            
            for (int i = 0; i < len; i++) {
                Point2D point = translator.geoToPixel(canvas, null, coords.get(i));
                xs[i] = (int) (point.getX() - canvas.x);
                ys[i] = (int) (point.getY() - canvas.y);
            }
            if (len >= 2) {
                renderLine(g, xs, ys);          
            }
            renderPoints(g, xs, ys);
        }
    }

    //TODO FP copied methods from PolygonDrawingRenederer
    //some AbstractDrawing Renderer could be created
    private void renderPoints(Graphics2D g, int[] xs, int[]  ys) {
        int len = xs.length;

        g.setColor(Color.DARK_GRAY);
        for (int i = 0; i < len - 1; i++) {
            g.fillRect(xs[i] - 1, ys[i] - 1, 3, 3);
        }
        // alt. style for last point:
        g.setColor(Color.YELLOW);
        g.fillRect(xs[len - 1] - 1, ys[len - 1] - 1, 3, 3);
    }

    private void renderLine(Graphics2D g, int[] xs, int[]  ys) {
        g.setColor(Color.RED);
        g.drawPolyline(xs, ys, xs.length);
    }
    
    private void renderGeometry(Graphics2D g, Geometry geometry, Color color, Rectangle canvas, GeoTranslator translator) {
        int len = geometry.getNumPoints();
        if (len > 2) {
            g.setColor(color);
            int xs[] = new int[len];
            int ys[] = new int[len];
            for (int i = 0; i < len; i++) {
                Point2D point2D = translator.geoToPixel(canvas, geometry.getEnvelope(), geometry.getCoordinates()[i]);
                xs[i] = (int) (point2D.getX() - canvas.getBounds().getX());
                ys[i] = (int) (point2D.getY() - canvas.getBounds().getY());
            }
            g.fillPolygon(xs, ys, len);
            
            // show geometry outline points;
            // make them a little larger than the split line points to show where they coincide:
            g.setColor(new Color(255, 255, 255, 191));
            for (int i = 0; i < len; i++) {
                g.fillRect(xs[i] - 2, ys[i] - 2, 5, 5);
            }
        }
    }    
        
}
